# Copyright 2024 DEViantUa <t.me/deviant_ua>
# All rights reserved.

from pydantic import BaseModel
from typing import List, Optional,Final

from ..tools.ukrainization import TranslateDataManager

UA_LANG = False
MAIN_LINK: Final[str] = "https://raw.githubusercontent.com/Mar-7th/StarRailRes/master/{icon}"


def hex_to_rgba(hex_code):
    hex_code = hex_code.strip('#')
    
    red = int(hex_code[0:2], 16)
    green = int(hex_code[2:4], 16)
    blue = int(hex_code[4:6], 16)
    
    alpha = 255 
    
    return red, green, blue, alpha

class MemoryInfo(BaseModel):
    level: Optional[int]
    chaos_id: Optional[int]
    chaos_level: Optional[int]
    
class SpaceInfo(BaseModel):
    memory_data: Optional[MemoryInfo]
    universe_level: Optional[int]
    light_cone_count: Optional[int]
    avatar_count: Optional[int]
    achievement_count: Optional[int]

class Avatar(BaseModel):
    id: str
    name: str
    icon: str

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.icon = MAIN_LINK.format(icon = self.icon)
    
        if UA_LANG:
            self.name = TranslateDataManager._data.avatar.get(self.id, self.name)
        
class Player(BaseModel):
    uid: Optional[str]
    nickname: Optional[str]
    level: int
    world_level: int
    friend_count: int
    avatar: Avatar
    signature: Optional[str]
    is_display: bool
    space_info: Optional[SpaceInfo]

class RelicMainAffix(BaseModel):
    type: Optional[str]
    field: Optional[str]
    name: Optional[str]
    icon: Optional[str]
    value: float
    display: Optional[str]
    percent: bool
    
    
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.icon = MAIN_LINK.format(icon = self.icon)
    
        if UA_LANG:
            self.name = TranslateDataManager._data.stats.get(self.type, self.name)

class RelicSubAffix(BaseModel):
    type: Optional[str]
    field: Optional[str]
    name: Optional[str]
    icon: Optional[str]
    value: float
    display: Optional[str]
    percent: bool
    count: int
    step: int

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.icon = MAIN_LINK.format(icon = self.icon)
    
        if UA_LANG:
            self.name = TranslateDataManager._data.stats.get(self.type, self.name)
            
class Relic(BaseModel):
    id: Optional[str]
    name: Optional[str]
    set_id: Optional[str]
    set_name: Optional[str]
    rarity: int
    level: int
    icon: Optional[str]
    main_affix: RelicMainAffix
    sub_affix: List[RelicSubAffix]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.icon = MAIN_LINK.format(icon = self.icon)
    
        if UA_LANG:
            self.set_name = TranslateDataManager._data.relict_sets.get(self.set_id, self.set_name)
            
class RelicSetProperties(BaseModel):
    type: Optional[str]
    field: Optional[str]
    name: Optional[str]
    icon: Optional[str]
    value: float
    display: Optional[str]
    percent: bool
    
    
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.icon = MAIN_LINK.format(icon = self.icon)
    
        if UA_LANG:
            self.name = TranslateDataManager._data.stats.get(self.type, self.name)

class RelicSet(BaseModel):
    id: Optional[str]
    name: Optional[str]
    icon: Optional[str]
    num: int
    desc: Optional[str]
    properties: List[RelicSetProperties]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.icon = MAIN_LINK.format(icon = self.icon)
    
        if UA_LANG:
            self.name = TranslateDataManager._data.relict_sets.get(self.id, self.name)
            
class LightConeAttributes(BaseModel):
    field: Optional[str]
    name: Optional[str]
    icon: Optional[str]
    value: float
    display: Optional[str]
    percent: bool
    
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.icon = MAIN_LINK.format(icon = self.icon)
    
        if UA_LANG:
            self.name = TranslateDataManager._data.stats.get(self.field, self.name)

class Path(BaseModel):
    id: Optional[str]
    name: Optional[str]
    icon: Optional[str]
    
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.icon = MAIN_LINK.format(icon = self.icon)
    
        if UA_LANG:
            self.name = TranslateDataManager._data.paths.get(self.id, self.name)

class LightCone(BaseModel):
    id: Optional[str]
    name: Optional[str]
    rarity: int
    rank: int
    level: int
    promotion: int
    icon: Optional[str]
    preview: Optional[str]
    portrait: Optional[str]
    path: Path
    attributes: List[LightConeAttributes]
    properties: List[LightConeAttributes]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.icon = MAIN_LINK.format(icon = self.icon)
        self.preview = MAIN_LINK.format(icon = self.preview)
        self.portrait = MAIN_LINK.format(icon = self.portrait)
    
        if UA_LANG:
            self.name = TranslateDataManager._data.weapons.get(self.id, self.name)
            
class ColorElement(BaseModel):
    hex: Optional[str]
    rgba: Optional[tuple]
     
class Element(BaseModel):
    id: Optional[str]
    name: Optional[str]
    color: Optional[str]
    icon: Optional[str]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.icon = MAIN_LINK.format(icon = self.icon)
        self.color = ColorElement(hex = self.color, rgba = hex_to_rgba(self.color))
        
        if UA_LANG:
            self.name = TranslateDataManager._data.element.get(self.id, self.name)
               
class Skill(BaseModel):
    id: Optional[str]
    name: Optional[str]
    level: int
    max_level: int
    element: Optional[Element]
    type: Optional[str]
    type_text: Optional[str]
    effect: Optional[str]
    effect_text: Optional[str]
    simple_desc: Optional[str]
    desc: Optional[str]
    icon: Optional[str]
    
class SkillTree(BaseModel):
    id: Optional[str]
    level: int
    anchor: Optional[str]
    max_level: int
    icon: Optional[str]
    parent: Optional[str]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.icon = MAIN_LINK.format(icon = self.icon)
        
class CharacterAttributes(BaseModel):
    field: Optional[str]
    name: Optional[str]
    icon: Optional[str]
    value: float
    display: Optional[str]
    percent: bool
    
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.icon = MAIN_LINK.format(icon = self.icon)

        if UA_LANG:
            self.name = TranslateDataManager._data.stats.get(self.field, self.name)
            
class Character(BaseModel):
    id: Optional[str]
    name: Optional[str]
    rarity: int
    rank: int
    level: int
    promotion: int
    icon: Optional[str]
    preview: Optional[str]
    portrait: Optional[str]
    rank_icons: list
    path: Path
    element: Optional[Element]
    skills: List[Skill]
    skill_trees: List[SkillTree]
    light_cone: Optional[LightCone]
    relics: Optional[List[Relic]]
    relic_sets: Optional[List[RelicSet]]
    attributes: List[CharacterAttributes]
    additions: List[CharacterAttributes]
    properties: List[CharacterAttributes]
    pos: list
    
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.icon = MAIN_LINK.format(icon = self.icon)
        self.preview = MAIN_LINK.format(icon = self.preview)
        self.portrait = MAIN_LINK.format(icon = self.portrait)
        self.pos = self.pos[0]
        new_rank_icons = []
        for key in self.rank_icons:
            new_rank_icons.append(MAIN_LINK.format(icon = key))
        self.rank_icons = new_rank_icons
        
        if UA_LANG:
            self.name = TranslateDataManager._data.avatar.get(self.id, self.name)
    
class MiHoMoApi(BaseModel):
    player: Player
    characters: List[Character]



# Copyright 2024 DEViantUa <t.me/deviant_ua>
# All rights reserved.


import aiohttp
import json

from pathlib import Path
from .src import utils

def decrypt_url(encrypted_url, key):
    decrypted = ''.join(chr(ord(c) ^ key) for c in encrypted_url)
    return decrypted

n = "FODFCDE"
r = "kLMHBJpWBQqBJOgJP@LQGaLW"
k = "5ceb5d1b20945b82dd21c7`g2909g1e4711d802b"


_LINK_SCORE = "https://raw.githubusercontent.com/"+decrypt_url(n,42)+"/"+decrypt_url(r,35)+"/"+"main"+"/generate/weight.json"
_LINK_DATA = "https://raw.githubusercontent.com/"+decrypt_url(n,42)+"/"+decrypt_url(r,35)+"/"+"main"+"/generate/{name}.json"
_LINK_SCORE_V2 = "https://raw.githubusercontent.com/"+decrypt_url(n,42)+"/StarRailScore/master/score.json"

_PATH = Path(__file__).parent /"src"/"assets"

_PATH_FILE_NAME = [
    "max",
    "relic_id",
    "rolls",
    "score"
]
async def get_score(link):
    async with aiohttp.ClientSession() as session:
        async with session.get(link) as response:
            data = await response.json(content_type='text/plain')
            return data



def open_score(name):
    with open(_PATH/f"{name}.json", 'r', encoding="utf-8") as file:
        data = json.load(file)

    return data

def save(name,data):
    with open(_PATH/f"{name}.json", "w", encoding="utf-8") as file:
        json.dump(data, file, indent=4, ensure_ascii=False)

class Calculator:
    def __init__(self, data) -> None:
        self.data = data
        self.score = open_score("score")
        self.rolls = open_score("rolls")
        self.relict_id = open_score("relic_id")
        self.max = open_score("max")

        self.result = {
            "score": {},
            "total_score": {"count": 0, "rank": {"name": "N/A", "color": (255,255,255)}},
            "bad": []
        }
    
    def get_rolls(self,rarity, stats):
        if rarity < 3:
            return 0
        low = self.rolls[stats.type][str(rarity)][0]
        mid = self.rolls[stats.type][str(rarity)][1]
        high = self.rolls[stats.type][str(rarity)][2]
        value = stats.value
        result = None
        max_margin = 100
        for i in range(rarity + 1):
            for j in range(rarity + 1):
                for k in range(rarity + 1):
                    if i + j + k > rarity + 1:
                        break
                    value_sum = i * low + j * mid + k * high
                    if abs(value_sum - value) < max_margin:
                        max_margin = abs(value_sum - value)
                        result = i+j+k 
        return result
    
    async def get_relic_score(self, chara_id, relic_json):
        result_json = {}
        
        main_weight = self.score[chara_id]["main"][self.relict_id.get(relic_json.id, relic_json.id)[-1]][relic_json.main_affix.type]
        main_affix_score = (relic_json.level + 1) / 16 * main_weight
        result_json["main_formula"] = f'{round((relic_json.level + 1) / 16 * 100, 1)}×{main_weight}={main_affix_score * 100}'
        
        sub_affix_formulas = []
        dont_sub = []
        for sub_affix_json in relic_json.sub_affix:
            if self.score[chara_id]["weight"][sub_affix_json.type] <= 0.3:
                dont_sub.append(sub_affix_json.type)
            sub_affix_formulas.append(f'{round(sub_affix_json.value / self.max[sub_affix_json.type] * 100, 1)}×{round(self.score[chara_id]["weight"][sub_affix_json.type], 1)}')
        sub_affix_score = sum((sub_affix_json.value / self.max[sub_affix_json.type]) * self.score[chara_id]["weight"][sub_affix_json.type] for sub_affix_json in relic_json.sub_affix)

        result_json["score"] = main_affix_score * 0.2 + sub_affix_score * 0.2
        result_json["sub_formulas"] = sub_affix_formulas

        return result_json,dont_sub

    async def start(self):
        
        if not self.data.id in self.score:
            await self.update_score(self.data.id)
            self.score = open_score("score")
        if not self.data.id in self.score:
            return self.result
        for key in self.data.relics:
            relic_score_json, bad = await self.get_relic_score(self.data.id,key)
            self.result["bad"] = list(set(self.result["bad"] + bad))
            relic_score = round(relic_score_json["score"] * 100, 1)
            self.result["total_score"]["count"] += relic_score
            self.result["score"][key.id] = {"count": relic_score, "rolls": {}, "rank": {"name": utils.get_relic_score_text(relic_score), "color": utils.get_relic_score_color(relic_score)}}
            for sub in key.sub_affix:
                rolls = self.get_rolls(key.rarity, sub)
                self.result["score"][key.id]["rolls"][sub.type] = rolls
        
        for key in self.result["score"]:
            self.result["total_score"]["count"] += self.result["score"][key]["count"]
        
        self.result["total_score"]["count"] = round(self.result["total_score"]["count"] /2, 1)
        self.result["total_score"]["rank"] = {"name": utils.get_relic_full_score_text(self.result["total_score"]["count"]), "color": utils.get_total_score_color(self.result["total_score"]["count"])}
         
        return self.result

        
    async def update_score(self, charter_id = None):
        
        for key in _PATH_FILE_NAME:
            if key == "score":
                data = await get_score(_LINK_SCORE)
                if not charter_id is None:
                    if data.get(str(charter_id)) is None:
                        data = await get_score(_LINK_SCORE_V2)
            else:
                data = await get_score(_LINK_DATA.format(name=key))
            save(key,data)


# Copyright 2024 DEViantUa <t.me/deviant_ua>
# All rights reserved.

from PIL import Image
import os
import threading
from pathlib import Path
from io import BytesIO

from .http import AioSession
from .cashe import Cache

lock = threading.Lock()

_caches = Cache.get_cache()

assets = Path(__file__).parent.parent / 'assets'

_BASE_URL = 'https://raw.githubusercontent.com/DEViantUA/StarRailCardData/main/asset/'

font = str(assets / 'font' / 'font_hsr.ttf')


def determine_font_path_automatically(font_file = 'Times New Roman.ttf'):
    font_dirs = [
        '/usr/share/fonts',          
        '/usr/local/share/fonts',    
        '/Library/Fonts',           
    ]
        
    for font_dir in font_dirs:
        font_path = os.path.join(font_dir, font_file)
        if os.path.isfile(font_path):
            return font_path

    return None

async def change_font(font_path = None):
    global font
    if font_path is None:
        font = str(assets / 'font' / 'font_hsr.ttf')
    else:
        font_path = os.path.abspath(font_path)
        if os.path.isfile(font_path):
            font = font_path 
        else:
            font_path = determine_font_path_automatically(font_path)
            if font_path is None:
                font = str(assets / 'font' / 'font_hsr.ttf')


total_style = {
    'g_five': 'teample_two_new/stars/g_five.png',
    'g_four': 'teample_two_new/stars/g_four.png',
    'g_three': 'teample_two_new/stars/g_three.png',
    'g_two': 'teample_two_new/stars/g_two.png',
    'g_one': 'teample_two_new/stars/g_one.png',
    
    'strs_1': 'stars/strs_1.png',
    'strs_2': 'stars/strs_2.png',
    'strs_3': 'stars/strs_3.png',
    'strs_4': 'stars/strs_4.png',
    'strs_5': 'stars/strs_5.png',
    
    'stars_v_5': 'three/stars/stars_5.png',
    'stars_v_4': 'three/stars/stars_4.png',
    'stars_v_3': 'three/stars/stars_3.png',
    'stars_v_2': 'three/stars/stars_2.png',
    'stars_v_1': 'three/stars/stars_1.png',
    
    "Knight": 'teample_two_new/path/Knight.png',
    "Mage": 'teample_two_new/path/Mage.png',
    "Priest": 'teample_two_new/path/Priest.png',
    "Rogue": 'teample_two_new/path/Rogue.png',
    "Shaman": 'teample_two_new/path/Shaman.png',
    "Warlock": 'teample_two_new/path/Warlock.png',
    "Warrior": 'teample_two_new/path/Warrior.png',
    
    'relict_backgroundl_score_line': 'teample_two_new/relict/backgroundl_score_line.png',
    'relict_full_score_line': 'teample_two_new/relict/full_score_line.png',
    
    'light_cone_ups': 'teample_two_new/light_cone/ups.png',
    
    'LOGO_GIT': 'teample_two_new/LOGO.png',
    'LOGO_GIT_INV': 'three/LOGO.png',
    'seeleland_v2': "seeleland_v2.png"
}

relict_score = {
    'background_default': 'teample_two_new/background/background_default.png',
    'background_line': 'teample_two_new/background/line.png',
    'background_maska_art': 'teample_two_new/background/maska_art.png',
    'background_maska_blur': 'teample_two_new/background/maska_blur.png',
    'background_overlay': 'teample_two_new/background/overlay.png',
    'background_shadow': 'teample_two_new/background/shadow.png',
    
    
    'light_cone_frame': 'teample_two_new/light_cone/frame.png',
    'light_cone_frame_five': 'teample_two_new/light_cone/frame_five.png',
    'light_cone_frame_four': 'teample_two_new/light_cone/frame_four.png',
    'light_cone_frame_three': 'teample_two_new/light_cone/frame_three.png',
    'light_cone_stats': 'teample_two_new/light_cone/stats.png',
        
    'relict_backgroundl_score_line': 'teample_two_new/relict/backgroundl_score_line.png',
    'relict_full_score_line': 'teample_two_new/relict/full_score_line.png',
    'relict_count_sets': 'teample_two_new/relict/count_sets.png',
    'relict_frame_line': 'teample_two_new/relict/frame_line.png',    
    'relict_line': 'teample_two_new/relict/line.png',
    'relict_frame': 'teample_two_new/relict/relict_frame.png',
    'relict_maska': 'teample_two_new/relict/relict_maska.png',
    'relict_score_frame': 'teample_two_new/relict/score_frame.png',
    'none_relict': 'teample_two_new/relict/none_relict.png',

    'path_count': 'teample_two_new/path/count.png',
    'path_closed_main': 'teample_two_new/path/closed_main.png',
    'path_closed_dop': 'teample_two_new/path/closed_dop.png',
}

ticket = {
    "background_line": 'three/bg/art_frame.png',
    "background_shadow": 'three/bg/shadow_art.png',
    "background_overlay": 'three/bg/overlay.png',
    
    'shadow_3_light_cone': 'three/light_cones/3_shadow_lc.png',
    'star_3_frame_light_cone': 'three/light_cones/3_star_frame_lc.png',
    'shadow_4_light_cone': 'three/light_cones/4_shadow_lc.png',
    'star_4_frame_light_cone': 'three/light_cones/4_star_frame_lc.png',
    'shadow_5_light_cone': 'three/light_cones/5_shadow_lc.png',
    'star_5_frame_light_cone': 'three/light_cones/5_star_frame_lc.png',
    'blic_light_cones': 'three/light_cones/blic.png',
    'frame_light_cones': 'three/light_cones/frame_lc.png',
    'maska_light_cones': 'three/light_cones/maska_lc.png',
    'stats_light_cones': 'three/light_cones/stats.png',
    
    
    'relict_count': 'three/relict/count.png',
    'relict_count_lvl': 'three/relict/count_lvl.png',
    'relict_background': 'three/relict/relict_background.png',
    'relict_maska': 'three/relict/relict_maska.png',
    'relict_score_frame': 'three/relict/score_frame.png',
    'none_relict': 'three/relict/none.png',
    'relict_icon_sets': 'three/relict/icon_sets.png',
    'relict_count_sets': 'three/relict/count_sets.png',
    
    "closed_main": 'three/skills/closed_main.png',
    "main_skills": 'three/skills/main.png',
    
    "frame_main": 'three/skills/frame_main.png',
    "bg_main": 'three/skills/main.png',
    
    "closed_trees": 'three/skills/closed_trees.png',
    "count_tree": 'three/skills/count_tree.png',
    "dop": 'three/skills/dop.png',
    "open_trees": 'three/skills/open_trees.png',

    "ON_const": 'three/const/ON.png',
    "OFF_const": 'three/const/OFF.png',
    "CLOSED_const": 'three/const/CLOSED.png',
}

profile_phone = {
    "bg_1": 'profile_phone/bg_1.png',
    "bg_2": 'profile_phone/bg_2.png',
    "bg_3": 'profile_phone/bg_3.png',
    "bg_5": 'profile_phone/bg_5.png',
    "desc_frame": 'profile_phone/desc_frame.png',
    "maska_prof_bg": 'profile_phone/maska_prof_bg.png',
    "menu": 'profile_phone/menu.png',
    "avatar_four": 'profile_phone/avatar_four.png',
    "avatar_five": 'profile_phone/avatar_five.png',
    'maska_character': 'profile_phone/maska_charter.png'
}

class ImageCache:
    
    _assets_dowload = False
    _mapping = {}
            
    @classmethod
    async def set_assets_dowload(cls, dowload = False):
        cls._assets_dowload = dowload
    
    @classmethod
    def set_mapping(cls,style):
        if style == 1:
            cls._mapping = relict_score
        elif style == 2:
            cls._mapping = ticket
        elif style == 3:
            cls._mapping = profile_phone
        
    @classmethod
    async def _load_image(cls, name):
        
        try:
            image = _caches[name]
        except KeyError:
            try:
                _caches[name] = image = Image.open(assets / name)
                return _caches[name]
            except Exception as e:
                pass
        
        try:
            _caches[name] = image = Image.open(assets / name)
            return _caches[name]
        except Exception as e:
            pass
        
        url = _BASE_URL + name
        if url in _caches:
            return _caches[name]
        else:
            image_data = await AioSession.get(url, response_format= "bytes")
            image = Image.open(BytesIO(image_data))
            _caches[name] = image
        
        if cls._assets_dowload:
            file_path = assets / name
            file_path.parent.mkdir(parents=True, exist_ok=True)
            image.save(str(assets / name))
        
        return image

    async def __getattr__(cls, name):
        if name in cls._mapping:
            return await cls._load_image(cls._mapping[name])
        else:
            if name in total_style:
                return await cls._load_image(total_style[name]) 
            else:
                raise AttributeError(f"'{cls.__class__.__name__}' object has no attribute '{name}'")
        
    async def download_icon_stats(self, prop_id):
        if 'icon_stats' in self.mapping:
            url = self.mapping['icon_stats'].format(prop_id=prop_id)
            full_url = _BASE_URL + url
            if full_url in _caches:
                return _caches[full_url].copy()
            else:
                image_data = await self.download_image(full_url)
                image = Image.open(image_data)
                _caches[full_url] = image
                return image.copy()
        else:
            raise AttributeError(f"'{self.__class__.__name__}' object has no attribute 'icon_stats'")

    async def download_icon_constant(self, element, unlock, resizes = None):
        if 'icon_const_unlock' in self.mapping and "icon_const_lock" in self.mapping:
            if unlock:
                url = self.mapping['icon_const_unlock'].format(element=element.upper())
            else:
                url = self.mapping['icon_const_lock'].format(element=element.upper())
            full_url = _BASE_URL + url
            key = (full_url, resizes, unlock)
            if key in _caches:
                return _caches[key].copy()
            else:
                image_data = await self.download_image(full_url)
                image = Image.open(image_data)
                if not resizes is None:
                    image = image.resize(resizes)
                    
                _caches[full_url] = image
                return image.copy()
        else:
            raise AttributeError(f"'{self.__class__.__name__}' object has no attribute 'icon_stats'")


# Copyright 2024 DEViantUa <t.me/deviant_ua>
# All rights reserved.

import asyncio
import io
from PIL import ImageDraw,Image,ImageChops, ImageSequence

from ..tools import git, options
from ..tools.calculator import stats
from ..model.style import Ticket

from ..tools import pill

_DEFAULT_SCORE = {'count': 0, 
                  'rolls': {}, 
                  'rank': {'name': 'N/A', 
                           'color': (255, 255, 255, 255)
                    }
}

_of = git.ImageCache()
_of.set_mapping(2)


class Creat:
    def __init__(self, data, lang, art, hide_uid, uid, seeleland, remove_logo) -> None:
        self.data = data
        self.remove_logo = remove_logo
        self.lang = lang
        self.art = art
        self.hide_uid = hide_uid
        self.uid = uid
        self.seeleland = seeleland
        self.totall_eff = 0
        self.seeleland = seeleland
        self.element_color = self.data.element.color.rgba
        self.gif = False
        self.GIFT_BG = []
        
    async def creat_bacground(self):
        self.background = Image.new(Ticket.RGBA, Ticket.bacground_size, (0,0,0,0))
        background_splash = Image.new(Ticket.RGBA, Ticket.background_splash_size, Ticket.bacground_default_color)
        background_dark = Image.new(Ticket.RGBA, Ticket.background_splash_size, (0,0,0,100))       
        
        line = await _of.background_line
        logo = await _of.LOGO_GIT_INV
        shadow = await _of.background_shadow
        splash = True
        background_overlay = await _of.background_overlay
        
        if self.art:
            user_image = await pill.get_centr_size(Ticket.art_size, self.art)
            self.background = pill.grandiend_v2.GrandientBackground(user_image,(1,Ticket.bacground_size[1])).start(left= True,overlay_add = False)
            line,self.element_color = await pill.recolor_image(line, self.element_color[:3], light = True)
            
                
            self.background = self.background.resize(Ticket.bacground_size)
            position_art = Ticket.position_art
            splash = False
            self.background = ImageChops.soft_light(self.background, background_overlay.convert(Ticket.RGBA))
            ll = await pill.light_level(self.element_color)
            if ll > 0.6:
                self.background.alpha_composite(background_dark)
        else:
            self.background = Image.new(Ticket.RGBA, Ticket.bacground_size, Ticket.bacground_default_color)
            user_image = await pill.get_centr_scale(Ticket.splash_size, await pill.get_dowload_img(self.data.portrait))
            position_art = Ticket.position_splash_art
            
        
        
        self.background.alpha_composite(user_image.convert(Ticket.RGBA),position_art)
        if splash:
            self.background.alpha_composite(background_splash)
        self.background.alpha_composite(line,Ticket.position_line)
        self.background.alpha_composite(shadow,Ticket.position_shadow)
        if not self.remove_logo:
            self.background.alpha_composite(logo,(1845,0))
        
        if not self.hide_uid:
            uid = Image.new(Ticket.RGBA, Ticket.bacground_size, (0,0,0,0))
            d = ImageDraw.Draw(uid)
            
            font_10 = await pill.get_font(Ticket.font_uid)
            d.text((0,0), f"UID: {self.uid}", font=font_10, fill=(255, 255, 255, 100))
            self.background.alpha_composite(uid,Ticket.position_uid)
    
    async def creat_light_cone(self):
        if self.data.light_cone is None:
            return Image.new(Ticket.RGBA, (0, 0), (0, 0, 0, 0))
        
        maska = await _of.maska_light_cones
        frame_light_cones = await _of.frame_light_cones
        blic_light_cones = await _of.blic_light_cones
        
        light_cone_holst = Image.new(Ticket.RGBA, Ticket.lc_bacgrount_size, (0, 0, 0, 0))
        light_cone_holst_image = Image.new(Ticket.RGBA, Ticket.lc_bacgrount_size, (0, 0, 0, 0))

        image = await pill.get_dowload_img(self.data.light_cone.portrait, size=Ticket.lc_image_size)
        light_cone_holst_image.alpha_composite(image, Ticket.lc_image_position)

        light_cone_holst.paste(light_cone_holst_image, (0, 0), maska.convert("L"))

        light_cone_holst_image = Image.new(Ticket.RGBA, Ticket.lc_bacgrount_size, (0, 0, 0, 0))
        
        shadow, frame, self.color = await pill.get_resurs_light_cones(self.data.light_cone.rarity)
        
        light_cone_holst_image.alpha_composite(shadow)
        light_cone_holst_image.alpha_composite(frame, Ticket.lc_frame_position)
        light_cone_holst_image.alpha_composite(light_cone_holst)
        light_cone_holst_image.alpha_composite(frame_light_cones)
        light_cone_holst_image.alpha_composite(blic_light_cones)
        light_cone_holst_image.alpha_composite(frame)

        
        background_stat = await _of.stats_light_cones
        background_stat = background_stat.copy()
        font_size = 29
        font_color = (193, 162, 103, 255)

        font = await pill.get_font(font_size)

        d = ImageDraw.Draw(background_stat)
        rank = options.ups(self.data.light_cone.rank)
        x = int(font.getlength(rank) / 2)
        d.text((17 - x, 2), rank, font=font, fill=font_color)

        font = await pill.get_font(19)

        max_level = options.max_lvl(self.data.light_cone.promotion)

        d.text((40,9), f"{self.lang.lvl}: {self.data.light_cone.level}/{max_level}", font=font, fill=(255, 255, 255, 255))
        
        d.text((49, 53), self.data.light_cone.attributes[0].display, font=font, fill=(255, 255, 255, 255)) #HP
        d.text((152, 53), self.data.light_cone.attributes[1].display, font=font, fill=(255, 255, 255, 255)) #ATK
        d.text((49, 100), self.data.light_cone.attributes[2].display, font=font, fill=(255, 255, 255, 255)) #DEF
        
        stars = await options.get_stars(self.data.light_cone.rarity,type=3)
        
        name_light_cone = await pill.create_image_with_text(self.data.light_cone.name, 24, max_width=230, color=(255, 255, 255, 255))
        line = Image.new("RGBA", (1, name_light_cone.size[1] + 2), self.color)

        self.light_cone_background = Image.new("RGBA", (447, 255), (0, 0, 0, 0))
        self.light_cone_background.alpha_composite(light_cone_holst_image.resize((183, 244)))
        self.light_cone_background.alpha_composite(stars,(0,6))
        self.light_cone_background.alpha_composite(line, (191, 19))
        self.light_cone_background.alpha_composite(name_light_cone, (200, 23))
        
        """if line.size[1] > 50:
            y = int(22 + line.size[1])
        else:
            y = -45"""
        
        y = int(22 + line.size[1])

        background_stat.save("BF.png")
        self.light_cone_background.alpha_composite(background_stat, (188, y))
    
    async def creat_relict(self,relict):
        
        font_15 = await pill.get_font(15)
        font_26 = await pill.get_font(26)
        font_14 = await pill.get_font(14)
        font_17 = await pill.get_font(17)
        
        background = Image.new(Ticket.RGBA, (394,115), (0,0,0,0))
        
        relict_background = await _of.relict_background
        relict_background = relict_background.copy()
        
        count = await _of.relict_count_lvl
        relict_maska = await _of.relict_maska
        relict_score_frame = await _of.relict_score_frame
        
        relict_background_icon = Image.new(Ticket.RGBA, (387,88), (0,0,0,0))
        image = await pill.get_dowload_img(relict.icon, size= (108,108))
        relict_background_icon.alpha_composite(image,(-6,-10))
        
        line = Image.new(Ticket.RGBA, (1,81), (255,255,255,255))
        
        relict_background.paste(relict_background_icon,(0,0),relict_maska.convert("L"))
        relict_background.alpha_composite(line,(117,4))
        relict_background.alpha_composite(line,(251,4))
        
        score = self.score_info["score"].get(str(relict.id),_DEFAULT_SCORE)
        main_stat_icon = await pill.get_dowload_img(relict.main_affix.icon, size= Ticket.relict_main_stat_icon_size)
        color = options.color_element.get(relict.main_affix.type, None)
        if not color is None:
            main_stat_icon = await pill.recolor_image(main_stat_icon, color[:3])
        
        stars = await options.get_stars(relict.rarity, type = 3)
        stars = stars.resize((25,82))
        
        relict_background.alpha_composite(main_stat_icon,Ticket.relict_main_stat_icon_position)
        relict_background.alpha_composite(count,(71,65))
        d = ImageDraw.Draw(relict_background)
        
        x = int(font_26.getlength(relict.main_affix.display))
        d.text((112-x, 36), relict.main_affix.display, font= font_26, fill= (0,0,0,255))
        d.text((113-x, 36), relict.main_affix.display, font= font_26, fill= (255,255,255,255))
        
        x = 93 - int(font_15.getlength(f"+{relict.level}")/2)
        d.text((x, 65), f"+{relict.level}", font= font_15, fill= (234,187,111,255))

        count = await _of.relict_count
        y_icon = 4
        x_icon = 118
        
        y_text = 16
        x_text = 211      
        
        p_roll_x = 222
        p_roll_y = 15
        
        eff = 0
        
        for i, k in enumerate(relict.sub_affix):
            count_draw  = count.copy()
            icon = await pill.get_dowload_img(k.icon, size=Ticket.relict_sub_icon_size)
            relict_background.alpha_composite(icon,(x_icon,y_icon))
            scoreR = score["rolls"].get(k.type,0)
            x = x_text - int(font_17.getlength(str(k.display)))
            
            if k.type in self.score_info["bad"]:
                d.text((x, y_text), str(k.display), font=font_17, fill= (255,255,255,255))
            else:
                eff += 1
                d.text((x, y_text), str(k.display), font=font_17, fill= options.color_scoreR.get(scoreR,(255,255,255,255)))
            
            
            draws = ImageDraw.Draw(count_draw)
            x = 12 - int(font_15.getlength(f"+{scoreR}")/2)
            draws.text((x, 0), f"+{scoreR}", font=font_15, fill= options.color_scoreR.get(scoreR,(255,255,255,255)))
            relict_background.alpha_composite(count_draw,(p_roll_x,p_roll_y))
            
            y_icon += 41
            y_text += 41
            p_roll_y += 41
            
            if i == 1:
                y_icon = 4
                x_icon = 251
                
                y_text = 16
                x_text = 344      
                
                p_roll_x = 355
                p_roll_y = 15
        
        
        background.alpha_composite(relict_score_frame)
        background.alpha_composite(relict_background, (7,17))
        background.alpha_composite(stars,Ticket.relict_position_star)
        
        d = ImageDraw.Draw(background)
        d.text((19, 1), "Score:", font=font_14, fill = (255,255,255,255))
        d.text((67, 1), str(score["count"]), font=font_14, fill = score["rank"]["color"])
        
        d.text((136, 1), "Rank:", font=font_14, fill = (255,255,255,255))
        d.text((178, 1), str(score["rank"]["name"]), font=font_14, fill = score["rank"]["color"])
        
        d.text((239, 1), "Eff Stat:", font=font_14, fill = (255,255,255,255))
        d.text((301, 1), str(eff), font=font_14, fill = options.color_scoreR.get(eff,(255,255,255,255)))
        
        self.totall_eff += eff
                
        return {"position": str(relict.id)[-1:] , "img": background}        

    
    async def creat_relict_sets(self):
        rel_set = options.calculator_relict_sets(self.data.relic_sets)

        self.background_sets = Image.new(Ticket.RGBA, Ticket.sets_background, (0,0,0,0))
        icons_sets = await _of.relict_icon_sets
        background_counts = await _of.relict_count_sets
        font = await pill.get_font(Ticket.setst_font)
        
        line_items = []
        i = 0
        for key in rel_set:
            sets = rel_set[key]
            holst_line = Image.new(Ticket.RGBA, Ticket.setst_line_size, (0,0,0,0))
            icon = await pill.get_dowload_img(sets["icon"], size = (33,33))
            icons_sets.alpha_composite(icon, (3,3))
            
            background_count = background_counts.copy()
            d = ImageDraw.Draw(background_count)
            d.text(Ticket.setst_count_position, str(sets["num"]), font=font, fill= Ticket.setst_name_color)
            
            d = ImageDraw.Draw(holst_line)
            sets_name_font,size = await pill.get_text_size_frame(sets["name"],15,280)
            if key[:1] == "1":
                if i != 0:
                    holst_line.alpha_composite(icons_sets, (356,0))
                    holst_line.alpha_composite(background_count,(0,10))
                    d.text((int(185-size/2), 12), sets["name"], font=sets_name_font, fill=Ticket.setst_name_color)
                    line_items.append({"setap": i, "line": holst_line})
                else:
                    holst_line.alpha_composite(icons_sets, (0,0))
                    holst_line.alpha_composite(background_count,(370,10))
                    d.text((int(185-size/2), 12), sets["name"], font=sets_name_font, fill=Ticket.setst_name_color)
                    line_items.append({"setap": i, "line": holst_line})
                i += 1
            else:
                holst_line.alpha_composite(icons_sets, (0,0))
                holst_line.alpha_composite(background_count,(370,10))
                d.text((int(185-size/2), 12), sets["name"], font=sets_name_font, fill=Ticket.setst_name_color)
                line_items.append({"setap": 3, "line": holst_line})
        
        
        
        for key in line_items:
            if key["setap"] == 0:
                self.background_sets.alpha_composite(key["line"],(0,0))
            elif key["setap"] == 3:
                self.background_sets.alpha_composite(key["line"],(413,507))
            else:
                self.background_sets.alpha_composite(key["line"],(0,45))
                
    async def get_score(self):
        self.score_info = await stats.Calculator(self.data).start()
    
    async def creat_stats(self):
        self.background_stats = Image.new(Ticket.RGBA, (434, 692), (0, 0, 0, 0))

        combined_attributes = {}
        dop = {}
        
        for attribute in self.data.attributes + self.data.additions:
            field = attribute.field
            if field in combined_attributes:
                combined_attributes[field].value += attribute.value
                dop[field]["dop"] = f"+{attribute.display}"
            else:
                dop[field] = {"main": attribute.display, "dop": 0}
                combined_attributes[field] = attribute

        dop = {key: value for key, value in dop.items() if value['dop'] != 0}
        
        result = []
                
        async for i, background in pill.creat_stats(combined_attributes, dop, 
                      Ticket.stat_font_dop,Ticket.stat_font,
                      Ticket.stat_line_size, Ticket.stat_name_size, Ticket.stat_max_width, Ticket.stat_icon_size,
                      Ticket.stat_icon_position, Ticket.stat_name_position, Ticket.stat_x_position,
                      self.element_color,Ticket.stat_value_font
                      ,Ticket.stat_y_no_dop,Ticket.stat_y_yes_dop,Ticket.stat_y_dop):
            result.append(background)
                
        if len(result) <= 1:
            x = 0 
        
        x = int(692 / (len(result)))
        position_line = 0
        for key in result:
            self.background_stats.alpha_composite(key, (0, position_line))
            position_line += x
                   
    async def creat_path(self):
        font = await pill.get_font(11)
        font_13 = await pill.get_font(13)

        self.main_bg =  Image.new("RGBA", (282,58), (0, 0, 0, 0))
        position_main = 0

        self.dop_bg = Image.new("RGBA", (399,124), (0, 0, 0, 0))

        position_dop = (
            (0,0),
            (192,0),
            (0,70),
            (192,70),
            (63,14),
            (100,14),
            (137,14),
            (255,14),
            (292,14),
            (329,14),
            (63,84),
            (100,84),
            (137,84),
            (255,84),
            (292,84),
            (329,84),
        )
        
        i = 0
        for key in self.data.skill_trees:
            if key.max_level == 1:

                if key.anchor in ["Point05","Point06","Point07","Point08"]:
                    icon = await pill.get_dowload_img(key.icon, size=(38,38))
                else:
                    icon = await pill.get_dowload_img(key.icon, size=(30,30))
                    
                if key.level == 0:
                    if key.anchor in ["Point05","Point06","Point07","Point08"]:
                        bg = await _of.dop
                        bg = bg.copy()
                        bg.alpha_composite(icon,(7,10))
                        closed_main = await _of.closed_main
                        bg.alpha_composite(closed_main,(0,0))
                    else:
                        bg = await _of.closed_trees
                        bg = await pill.recolor_image(bg.copy(), (0, 0, 0))
                        icon = await pill.recolor_image(icon,self.element_color[:3])
                        bg.alpha_composite(icon, (0,0))
                else:
                    if key.anchor in ["Point05","Point06","Point07","Point08"]:
                        bg = await _of.dop
                        bg = bg.copy()
                        bg.alpha_composite(icon,(7,10))
                    else:
                        bg = await _of.open_trees
                        bg = await pill.recolor_image(bg.copy(), self.element_color[:3])
                        icon = await pill.recolor_image(icon, (0, 0, 0))
                        bg.alpha_composite(icon, (0,0))
                self.dop_bg.alpha_composite(bg, position_dop[i])
                i += 1
            else:
                icon = await pill.get_dowload_img(key.icon, size=(43, 43))
                bg = await _of.bg_main
                frame_bg = await _of.frame_main
                bg = bg.copy()
                frame_bg = await pill.recolor_image(frame_bg.copy(),self.element_color[:3])
                bg.alpha_composite(frame_bg)
                
                count = await _of.count_tree
                count = await pill.recolor_image(count.copy(),self.element_color[:3])
                bg.alpha_composite(icon,(4,4))
                draw = ImageDraw.Draw(count)
                if key.anchor == 'Point01':
                    if key.max_level > 6:
                        key.level += 1
                else:
                    if key.max_level > 10:
                        key.level += 2
                x = int(font.getlength( f"{key.level}/{key.max_level}")/2)
                ll = await pill.light_level(self.element_color)
                if ll > 0.6:
                    draw.text((18-x,0), f"{key.level}/{key.max_level}", font=font_13, fill=(0, 0, 0, 255))
                else:
                    draw.text((18-x,0), f"{key.level}/{key.max_level}", font=font_13, fill=(255, 255, 255, 255))
                bg.alpha_composite(count,(4,44))
                self.main_bg.alpha_composite(bg,(position_main,0))
                position_main += 77


    async def creat_name(self):
        self.background_name = Image.new(Ticket.RGBA, Ticket.name_size, (0, 0, 0, 0))
        d = ImageDraw.Draw(self.background_name)
        names = await pill.create_image_with_text(self.data.name, Ticket.font_name_size , max_width=Ticket.name_with, color= Ticket.name_color)
        names_dark = await pill.recolor_image(names,(0,0,0))
        if names_dark.size[1] <= Ticket.name_h_max:
            self.background_name.alpha_composite(names_dark,(Ticket.name_position - 1,int(Ticket.name_h_max-names.size[1]/2)))
        else:
            self.background_name.alpha_composite(names_dark,(Ticket.name_position - 1,int(Ticket.name_h_min-names.size[1]/2)))
            
        if names.size[1] <= Ticket.name_h_max:
            self.background_name.alpha_composite(names,(Ticket.name_position,int(Ticket.name_h_max-names.size[1]/2)))
        else:
            self.background_name.alpha_composite(names,(Ticket.name_position,int(Ticket.name_h_min-names.size[1]/2)))
        
            
        font_17 = await pill.get_font(Ticket.font_name_level)
        max_level = options.max_lvl(self.data.promotion)
        level = f"{self.lang.lvl}: {self.data.level}/{max_level}"
        d.text((Ticket.position_name_level[0]-1, Ticket.position_name_level[1]), level, font=font_17, fill= (0,0,0,255))
        d.text(Ticket.position_name_level, level, font=font_17, fill= Ticket.color_name_level)
        starts = await options.get_stars(self.data.rarity)
        self.background_name.alpha_composite(starts,Ticket.position_name_star)   
    
    async def creat_constant(self):
        self.background_skills = Image.new(Ticket.RGBA, (381, 54), (0, 0, 0, 0))
        x = 0
        rank = self.data.rank
        closed = await _of.CLOSED_const
        closed = closed.resize((54,54))
        closed = await pill.recolor_image(closed.copy(), self.element_color[:3])
        for key in self.data.rank_icons[:rank]:
            bg = await _of.ON_const
            bg = await pill.recolor_image(bg.resize((54,54)).copy(),self.element_color[:3])
            icon = await pill.get_dowload_img(key, size=(43,43))
            icon = await pill.recolor_image(icon, self.element_color[:3])
            bg.alpha_composite(icon,(5,5))
            self.background_skills.alpha_composite(bg,(x,0))
            x += 65
        for key in self.data.rank_icons[rank:]:
            bg = await _of.OFF_const
            bg = bg.resize((54,54)).copy()
            icon = await pill.get_dowload_img(key, size=(43,43))
            icon = await pill.apply_opacity(icon, 0.3)
            bg.alpha_composite(icon,(5,5))
            bg.alpha_composite(closed,(0,0))
            self.background_skills.alpha_composite(bg,(x,0))
            x += 65
   
    
    async def creat_score_total(self):
        self.background_score = Image.new("RGBA",(442,37), (0,0,0,0))
        
        sclor_bg = await _of.relict_backgroundl_score_line
        sclor_bg = sclor_bg.copy()
        sclor_bg_color = await _of.relict_full_score_line
        sclor_bg_color = sclor_bg_color.copy()
        sclor_bg_color = await pill.recolor_image(sclor_bg_color, self.element_color[:3])
        
        result_percentage = (self.score_info["total_score"]['count'] / 260.53) * 100
        pixels_to_fill = int((result_percentage / 100)*599)
        if pixels_to_fill <= 0:
            pixels_to_fill = 1
        sclor_bg.alpha_composite(sclor_bg_color.resize((pixels_to_fill,16)))
        
        font_18 = await pill.get_font(18)
        
        d = ImageDraw.Draw(sclor_bg)
        x = 280 - int(font_18.getlength(f"{round(result_percentage,1)}%")/2)
        frame = await pill.get_colors(await pill.crop_image(sclor_bg), 15, common=True, radius=5, quality=800) 
        ll = await pill.light_level(frame)
        if ll > 0.6 or frame == (102, 53, 0):
            d.text((x, -1), f"{round(result_percentage,1)}%", font=font_18, fill = (0,0,0,0))
        else:
            d.text((x, -1), f"{round(result_percentage,1)}%", font=font_18, fill = (255,255,255,255))
        
        self.background_score.alpha_composite(sclor_bg.resize((394,13)),(0,24))
        
        font_16 = await pill.get_font(16)
        d = ImageDraw.Draw(self.background_score)
        
        d.text((178, -2), "Score:", font=font_16, fill = (255,255,255,255))
        d.text((233, -2), str(self.score_info["total_score"]['count']), font=font_16, fill = self.score_info["total_score"]["rank"]["color"])
        
        d.text((0, -2), "Summary Rank:", font=font_16, fill = (255,255,255,255))
        d.text((132, -2), str(self.score_info["total_score"]["rank"]["name"]), font=font_16, fill = self.score_info["total_score"]["rank"]["color"])
        
        d.text((287, -2), "Eff Stat:", font=font_16, fill = (255,255,255,255))
        d.text((366, -2), str(self.totall_eff), font=font_16, fill = options.color_scoreR.get(self.totall_eff,(255,255,255,255)))
    
    
    async def build_relict(self):
        self.background_relict = Image.new(Ticket.RGBA, (807,457), (0,0,0,0))
        none_relict = await _of.none_relict
        position = {
            "1": (0,0),
            "2": (0,115),
            "3": (0,237),
            "4": (0,352),
            "5": (413,185),
            "6": (413,300),
        }
        
        map = {"1": None, "2": None, "3": None, "4": None, "5": None, "6": None}
        
        for key in self.relict:
            self.background_relict.alpha_composite(key["img"], position.get(key["position"]))
            map[key["position"]] = 0
        
        for key in map:
            if map[key] is None:
                self.background_relict.alpha_composite(none_relict, position.get(key))
    
    
    async def creat_seeleland(self):
        self.seelelen = Image.new("RGBA",(1,1), (0,0,0,0))
        
        if self.seeleland:
            self.seelelen = await _of.seeleland_v2
            self.seelelen = self.seelelen.copy()
            font = await pill.get_font(15)
            data = await options.get_seeleland(self.uid, self.data.id)
            if data is None:
                self.seelelen = Image.new("RGBA",(1,1), (0,0,0,0))
                return None
            if not "rank" in data:
                self.seelelen = Image.new("RGBA",(1,1), (0,0,0,0))
                return None
            draw = ImageDraw.Draw(self.seelelen)
            draw.text((143, 7), str(data["sc"]), font=font, fill=(255, 255, 255, 255))
            draw.text((136, 24), f'{data["rank"][:-2]}..', font=font, fill=(255, 255, 255, 255))
            draw.text((127, 42), data["percrank"].replace("top ", ""), font=font, fill=(255, 255, 255, 255))
    
    async def build(self):
        bg = Image.new(Ticket.RGBA, Ticket.bacground_size, (0,0,0,0))
        
        bg.alpha_composite(self.light_cone_background.resize((302,182)),(29,35))
        bg.alpha_composite(self.background_relict,(29,310)) 
        bg.alpha_composite(self.background_sets,(29,222))
        bg.alpha_composite(self.background_stats,(884,56))
        bg.alpha_composite(self.main_bg,(500,68))
        bg.alpha_composite(self.dop_bg,(446,165))
        bg.alpha_composite(self.background_skills,(455,327))
        bg.alpha_composite(self.background_name,(1350,691))
        bg.alpha_composite(self.background_score,(442,436))
        bg.alpha_composite(self.seelelen,(1688,718)) 
        
        if self.gif:
            self.background = []
            for key in self.GIFT_BG:
                key.alpha_composite(bg)
                self.background.append(key.convert("RGB"))
        else:
            self.background.alpha_composite(bg)
        
    async def start(self):
        
        _of.set_mapping(2)
        
        if self.art:
            if "gif" in self.art:
                self.gif = True
            self.art = await pill.get_user_image(self.art)
            if self.gif:    
                n = 2           
                frame_count = 0
                color = True
                with io.BytesIO(self.art) as f:
                    self.art = Image.open(f)
                    for frame in ImageSequence.Iterator(self.art):
                        self.element_color = await pill.get_colors(frame.convert("RGBA"), 15, common=True, radius=5, quality=800)
                        if frame_count >= 50:
                            break
                        if frame_count % n != 0:
                            self.art = frame.convert("RGBA")
                            await self.creat_bacground()
                            self.GIFT_BG.append(self.background.copy())
                        frame_count += 1
            else:
                self.element_color = await pill.get_colors(self.art, 15, common=True, radius=5, quality=800)
                await self.creat_bacground()
        else:
            self.element_color = (255,213,167,255)
            await self.creat_bacground()
        
        await asyncio.gather(
            self.creat_light_cone(),
            self.creat_stats(),
            self.creat_name(),
            self.creat_constant(),
            self.creat_relict_sets(),
            self.get_score(),
            self.creat_path(),
            self.creat_seeleland()
        )
        relic_tasks = [self.creat_relict(key) for key in self.data.relics]
        self.relict = await asyncio.gather(*relic_tasks)
        
        await self.creat_score_total()
        
        await self.build_relict()
        await self.build()
        
        data = {
                "id": self.data.id,
                "name": self.data.name,
                "animation": self.gif,
                "rarity": self.data.rarity,
                "card": self.background,
                "size": Ticket.bacground_size,
                "color": self.element_color,
            }
            
        return data


# Copyright 2024 DEViantUa <t.me/deviant_ua>
# All rights reserved.

from ..tools import http, translator, ukrainization
from ..model import api_mihomo
from typing import Final

_API_MIHOMO: Final[str] = "https://api.mihomo.me/sr_info_parsed/{uid}?version=v2&lang={lang}&is_force_update={force_update}"


class ApiMiHoMo:
    def __init__(self,uid, lang = "en", force_update = False) -> None:
        self.force_update = force_update
        self.uid = uid
        api_mihomo.UA_LANG = False
        if lang == "ua":
            api_mihomo.UA_LANG = True
        self.lang = translator.SUPPORTED_LANGUAGES.get(lang, "en")
    
    async def get(self):
        data = await http.AioSession.get(_API_MIHOMO.format(uid = self.uid, lang = self.lang, force_update = self.force_update))
        if api_mihomo.UA_LANG:
            await ukrainization.TranslateDataManager().load_translate_data()
            
        return api_mihomo.MiHoMoApi(**data)


# Copyright 2024 DEViantUa <t.me/deviant_ua>
# All rights reserved.


from pydantic import BaseModel
from PIL import Image
from typing import List ,Optional,Final, Union

try:
    from moviepy.editor import ImageSequenceClip
except:
    pass

import numpy as np

from ..tools.ukrainization import TranslateDataManager
from ..tools.http import AioSession

UA_LANG = False
MAIN_LINK: Final[str] = "https://raw.githubusercontent.com/Mar-7th/StarRailRes/master/{icon}"

class MemoryInfo(BaseModel):
    level: Optional[int]
    chaos_id: Optional[int]
    chaos_level: Optional[int]
    
class SpaceInfo(BaseModel):
    memory_data: Optional[MemoryInfo]
    universe_level: Optional[int]
    light_cone_count: Optional[int]
    avatar_count: Optional[int]
    achievement_count: Optional[int]

class Avatar(BaseModel):
    id: str
    name: str
    icon: str

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        #self.icon = MAIN_LINK.format(icon = self.icon)
    
        if UA_LANG:
            self.name = TranslateDataManager._data.avatar.get(self.id, self.name)

class Player(BaseModel):
    uid: Optional[str]
    nickname: Optional[str]
    level: int
    world_level: int
    friend_count: int
    avatar: Avatar
    signature: Optional[str]
    is_display: bool
    space_info: Optional[SpaceInfo]

class Card(BaseModel):
    id: int
    name: Optional[str]
    rarity: int
    card: Union[Image.Image,list]
    animation: bool
    size: Optional[tuple]
    color: Optional[tuple]
    class Config:
        arbitrary_types_allowed = True
    
    async def get_info(self, lang = "en"):
        await AioSession.get_session()
        url = f"https://api.yatta.top/hsr/v2/{lang}/avatar/{self.id}"
        data = await AioSession.get(url, response_format = "json")
        data["data"]["icon"] = {"icon": f'https://api.yatta.top/hsr/assets/UI/avatar/medium/{data["data"]["icon"]}.png', 
                                "splash": f'https://api.yatta.top/hsr/assets/UI/avatar/large/{data["data"]["icon"]}.png',
                                "avatar": f'https://raw.githubusercontent.com/Mar-7th/StarRailRes/master/icon/avatar/{data["data"]["icon"]}.png'
        }        
        await AioSession.close_session()
        
        return data["data"]
    
    def show(self):
        if self.animation:
            self.card[0].show()
        else:
            self.card.show()
    
    def save_gif(self, method = "pillow", name_file = None, format = "gif", resize = None):
        if name_file is None:
            name_file = f"{self.id}_{self.rarity}"
        if isinstance(self.card, list):
            if not format in ["gif", "webp"]:
                format = "gif"
            if format == "webp":
                method = "pillow"
            
            if method.lower() == "pillow":
                self.card[0].save(f"{name_file}.{format}", format = format.capitalize(), save_all=True, append_images=self.card[1:], optimize=True, quality=100, duration=100, loop=0)
            else:
                if resize is None:
                    self.card = [np.array(frame) for frame in self.card]
                else:
                    self.card = [np.array(frame.resize(resize)) for frame in self.card]
                clip = ImageSequenceClip(self.card, fps=10)
                clip.write_gif(f"{name_file}.{format}", fps=10, loop=True, program='ffmpeg', fuzz=0, opt='OptimizeTransparency', logger = None )
        else:
            self.card.save(f"{name_file}.png")
        
class Setting(BaseModel):
    uid: int
    lang: Optional[str]
    hide_uid: bool
    save: bool
    force_update: bool
    style: int
    
class StarRail(BaseModel):
    settings: Setting
    player: Player
    card: Union[List[Card], Image.Image]
    character_id: list
    character_name: list
    
    class Config:
        arbitrary_types_allowed = True
    
    def __str__(self):
        return f"card={self.card} character_name = {self.character_name} character_id={self.character_id}"
    
    def get_charter(self, setting = False, name = False):
        if setting:
            card_ids = [str(card.id) for card in self.card]

            if name:
                return {name: id for id, name in zip(self.character_id, self.character_name) if id in card_ids}
            return {id: name for id, name in zip(self.character_id, self.character_name) if id in card_ids}
        
        if name:
            return {name: id for id, name in zip(self.character_id, self.character_name)}
        
        return {id: name for id, name in zip(self.character_id, self.character_name)}


# Copyright 2024 DEViantUa <t.me/deviant_ua>
# All rights reserved.

import random
import aiofiles
import io
from PIL import Image
import os
import datetime

from .git import ImageCache
from .http import AioSession

_git = ImageCache()

_DEFAULT_SCORE = {'count': 0, 
                  'rolls': {}, 
                  'rank': {'name': 'N/A', 
                           'color': (255, 255, 255, 255)
                    }
}

color_score  = {
    4: (255, 250, 141, 255),
    3: (252, 209, 124, 255),
    2: (184, 157, 103, 255),
    1: (170, 145, 98, 255),
    0: (255, 255, 255, 255)
}

color_scoreR = {
    4: (255, 250, 141, 255),
    3: (252, 209, 124, 255),
    2: (234, 207, 153, 255),  # Было (184, 157, 103, 255)
    1: (220, 195, 148, 255),  # Было (170, 145, 98, 255)
    0: (255, 255, 255, 255)   # Было (255, 255, 255, 255)
}


color_element = {
    'PhysicalAddedRatio': (255, 255, 255, 255),
    'PhysicalResistance': (255, 255, 255, 255),
    'FireAddedRatio': (248, 79, 54, 255),
    'FireResistance': (248, 79, 54, 255),
    'IceAddedRatio': (71, 199, 253, 255),
    'IceResistance': (71, 199, 253, 255),
    'ThunderAddedRatio': (136, 114, 241, 255),
    'ThunderResistance': (136, 114, 241, 255),
    'WindAddedRatio': (0, 255, 156, 255),
    'WindResistance': (0, 255, 156, 255),
    'QuantumAddedRatio': (28, 41, 186, 255),
    'QuantumResistance': (28, 41, 186, 255),
    'ImaginaryAddedRatio': (244, 210, 88, 255),
    'ImaginaryResistance': (244, 210, 88, 255)
}


color_lc_line = {
    "3": (150, 202, 255, 255),
    "4": (217, 150, 255, 255),
    "5": (255, 217, 144, 255),
}


async def get_charter_id(data):
    data = [value.strip() for value in data.split(',') if value.strip()]

    data = [value for value in data if value.isdigit()]
    
    if data == []:
        return None
    return data

async def style_setting(style, settings):
    if str(style) in ["1","2","3"]:
        return style, settings
    
    return 1, {}


def ups(x):
    if x == 5:
        return "V"
    elif x == 4:
        return "IV"
    elif x == 3:
        return "III"
    elif x == 2:
        return "II"
    elif x == 1:
        return "I"
    else:
        return "O"


def max_lvl(x):
    if x == 0:
        max = 20
    elif x == 1:
        max = 30
    elif x == 2:
        max = 40 
    elif x == 3:
        max = 50
    elif x == 4:
        max = 60
    elif x == 5:
        max = 70
    else:
        max = 80
    
    return max


async def get_character_art(character_art):
    processed_dict = {}
    for key, value in character_art.items():
        if isinstance(value, list):
            processed_dict[key] = random.choice(value)
        else:
            processed_dict[key] = value

    return processed_dict

async def get_stars(x, type = 1):
    if type == 1:
        if x == 5:
            return await _git.g_five
        elif x == 4:
            return await _git.g_four
        elif x == 3:
            return await _git.g_three
        elif x == 2:
            return await _git.g_two
        else:
            return await _git.g_one
    elif type == 2:
        if x == 1:
            return await _git.strs_1
        elif x == 2:
            return await _git.strs_2
        elif x == 3:
            return await _git.strs_3
        elif x == 4:
            return await _git.strs_4
        else:
            return await _git.strs_5
    elif type == 3:
        if x == 1:
            return await _git.stars_v_1
        elif x == 2:
            return await _git.stars_v_2
        elif x == 3:
            return await _git.stars_v_3
        elif x == 4:
            return await _git.stars_v_4
        else:
            return await _git.stars_v_5
        
def calculator_relict_sets(data):
    rel_set = {}
    for key in data:
        if key.id not in rel_set:
            if key.properties == []:
                rel_set[key.id] = {"num": int(key.num), "name": key.name, "icon": key.icon, "properties": None}
            else:
                rel_set[key.id] = {"num": int(key.num), "name": key.name, "icon": key.icon, "properties": {"icon": key.properties[0].icon, "display": key.properties[0].display}}
        else:
            rel_set[key.id]["num"] = int(key.num)
    
    return rel_set

async def get_background_path(path):
    if path == "Rogue":
        icon =  await _git.Rogue
    elif path == "Knight":
        icon =  await _git.Knight
    elif path == "Mage":
        icon =  await _git.Mage
    elif path == "Priest":
        icon =  await _git.Priest
    elif path == "Shaman":
        icon =  await _git.Shaman
    elif path == "Warlock":
        icon =  await _git.Warlock
    else:
        icon = await _git.Warrior
    
    return icon.copy()


async def get_seeleland(uid, charter_id):
    url = f"https://seeleland.azurewebsites.net/api/get_player_data?uid={uid}"
    data = await AioSession.get(url, response_format= "json")
    if data is None:
        return None
    for key in data:
        if key["k"] == str(charter_id):
            data = key.get("lb")
    
    if data != {} and data is not None:
        if type(data) == list:
            for key in data:
                if "lb" in key:
                    return key["lb"]["tutorial"]
        else:
            for key in data:
                return data[key]
    else:
        return None

async def save_card(uid, image_data, name):
    data = datetime.datetime.now().strftime("%d_%m_%Y %H_%M")
    path = os.getcwd()
    
    try:
        os.makedirs(f'{path}/RailCardImg/{uid}', exist_ok=True)
    except FileExistsError:
        pass
    
    file_name = f"{path}/RailCardImg/{uid}/{name}_{data}.png"
    
    async with aiofiles.open(file_name, 'wb') as file:
        if isinstance(image_data, Image.Image):
            img_bytes = io.BytesIO()
            image_data.save(img_bytes, format='PNG')
            await file.write(img_bytes.getvalue())



# Copyright 2024 DEViantUa <t.me/deviant_ua>
# All rights reserved.

SUPPORTED_LANGUAGES = {
    'cht': 'cht',
    'cn': 'cn',
    'de': 'de',
    'en': 'en',
    'es': 'es',
    'fr': 'fr',
    'id': 'id',
    'jp': 'jp',
    'kr': 'kr',
    'pt': 'pt',
    'ru': 'ru',
    'th': 'th',
    'vi': 'vi',
    'ua': 'ru'
}


translationLang = {"en":{"lvl": "LVL", "AR":"AR", "WL":"WL", "AC": "Achievements", "AB": "Abyss"}, 
        "ru": {"lvl": "Уровень", "AR":"РП", "WL":"УМ", "AC": "Достижения", "AB": "Бездна"},
        "ua": {"lvl": "Рівень", "AR": "РП", "WL": "РС", "AC": "Досягнення", "AB": "Безодня"},
        "vi": {'lvl': 'Cấp độ ', 'AR': 'AR', 'WL': 'WL', 'AC': 'Thành tích ', 'AB': 'Vực thẳm'},
        "th": {'lvl': 'ระดับ ', 'AR': 'AR', 'WL': 'WL', 'AC': ' ความสำเร็จ ', 'AB': 'Abyss'},
        "pt": {'lvl': 'Nível ', 'AR': 'AR', 'WL': 'WL', 'AC': ' Conquistas ', 'AB': 'Abismo'},
        "kr": {'lvl': '레벨 ', 'AR': 'AR', 'WL': 'WL', 'AC': '업적', 'AB': '어비스'},
        "jp": {'lvl': 'レベル ', 'AR': 'AR', 'WL': 'WL', 'AC': 'アチーブメント', 'AB': 'アビス'},
        "zh": {'lvl': '等级', 'AR': 'AR', 'WL': 'WL', 'AC': '成就总数', 'AB': '深境螺旋'},
        "cn": {'lvl': '等级', 'AR': 'AR', 'WL': 'WL', 'AC': '成就总数', 'AB': '深境螺旋'},
        "id": {'lvl': 'Level ', 'AR': 'AR', 'WL': 'WL', 'AC': ' Prestasi ', 'AB': ' Abyss'},
        "fr": {'lvl': 'Niveau ', 'AR': 'AR', 'WL': 'WL', 'AC': ' Réalisations ', 'AB': ' Abîme'},
        "es": {'lvl': 'Nivel ', 'AR': 'AR', 'WL': 'WL', 'AC': ' Logros ', 'AB': ' Abismo'},
        "de": {'lvl': 'Level ', 'AR': 'AR', 'WL': 'WL', 'AC': ' Erfolge ', 'AB': ' Abyss'},
        "chs": {'lvl': '等级', 'AR': 'AR', 'WL': 'WL', 'AC': '成就总数', 'AB': '深境螺旋'},
        "cht": {'lvl': '等級', 'AR': 'AR', 'WL': 'WL', 'AC': '成就總數', 'AB': '深境螺旋'},
    }


class Translator:
    def __init__(self,lang) -> None:
        self.lang = str(lang)
        
    def __getattr__(self,name):
        if self.lang  in translationLang:
            return translationLang[self.lang].get(name)
        else:
            raise AttributeError(f"'{type(self)}' object has no attribute '{self.lang}'")



# Copyright 2024 DEViantUa <t.me/deviant_ua>
# All rights reserved.

import asyncio
import random
from PIL import ImageDraw,Image,ImageChops
from ..tools import pill, git, options

_of = git.ImageCache()

class Creat:
    def __init__(self,profile,lang,img,hide,uid,background, remove_logo) -> None:
        self.remove_logo = remove_logo
        self.profile = profile
        self.lang = lang
        self.img = img
        self.hide = hide
        self.uid = uid
        self.background = background
    
    async def creat_background(self):
        self.background_profile = Image.new("RGBA", (828, 1078), (0,0,0,0))
        
        maska,frame = await asyncio.gather(_of.maska_prof_bg,_of.menu)
        if self.background is None:
            background_image = random.choice([await _of.bg_1, await  _of.bg_2, await  _of.bg_3, await  _of.bg_5])
            background_shadow = Image.new("RGBA", (828, 1078), (0,0,0,100))
        else:
            background_image = await pill.get_dowload_img(self.background)
            background_image = await pill.get_centr_size((828,1078),background_image)
            background_shadow = Image.new("RGBA", (828, 1078), (0,0,0,150))
        background_image = background_image.convert("RGBA")
        background_image.alpha_composite(background_shadow)
        self.background_profile.paste(background_image,(0,0),maska.convert("L"))
        self.background_profile.alpha_composite(frame)

    async def creat_charter(self,key):
        
        if key.rarity == 5:
            charter_profile = await _of.avatar_five
        else:
            charter_profile = await _of.avatar_four
        
        charter_profile = charter_profile.copy()
        
        if not self.img is None:
            if str(key.id) in self.img:
                url_id = self.img[str(key.id)]
            else:
                url_id = f'https://raw.githubusercontent.com/Mar-7th/StarRailRes/master/icon/avatar/{key.id}.png'
        else:
            url_id = f'https://raw.githubusercontent.com/Mar-7th/StarRailRes/master/icon/avatar/{key.id}.png'
            
        splash,mask = await asyncio.gather(pill.get_dowload_img(url_id),_of.maska_character)
        splash = await pill.get_centr_size((109,109),splash)
        charter_profile.paste(splash,(16,28),mask.convert("L"))
        
        name = await pill.create_image_with_text(key.name,16, max_width=123, color=(255, 255, 255, 255)) 
        
        charter_profile.alpha_composite(name,(136,int(49-name.size[1]/2)))       

        max_level = options.max_lvl(key.promotion)
        level = f"{self.lang.lvl}: {key.level}/{max_level}"
        
        element_icon = await pill.get_dowload_img(key.element.icon, size=(28,28))
        path_icon = await pill.get_dowload_img(key.path.icon, size=(28,28))
        starts = await options.get_stars(key.light_cone.rarity)
        
        charter_profile.alpha_composite(path_icon,(263,28))
        charter_profile.alpha_composite(element_icon,(293,28))
        charter_profile.alpha_composite(starts.resize((85,18)),(224,114))
        
        
        d = ImageDraw.Draw(charter_profile)
        d.text((19,7), level, font= self.font_charter, fill=(255,255,255,255))
        
        
        if not key.light_cone is None:
            icon = await pill.get_dowload_img(key.light_cone.icon, size = (66,66))
            charter_profile.alpha_composite(icon,(136,70))
            d.text((201,95), f"{self.lang.lvl}: {key.light_cone.level}/{options.max_lvl(key.light_cone.promotion)}", font=self.font_charter, fill=(255, 255, 255, 255))

            
            background = await _of.light_cone_ups
            background = background.copy()
            
            d = ImageDraw.Draw(background)
            font_12 = await pill.get_font(12)
            up = options.ups(key.light_cone.rank)
            x = int(font_12.getlength(str(up))/2)
            d.text((10-x, 4), up, font= font_12, fill=(255, 217, 144, 255))
            
            charter_profile.alpha_composite(background.resize((17,17)), (202,114))
        
        return charter_profile
    
    async def get_charter(self):
        task = []
        for key in self.profile.characters:
            task.append(self.creat_charter(key))
        
        self.charter = await asyncio.gather(*task)
    
    async def creat_avatar(self):
        self.background_profile_avatar = Image.new("RGBA", (625, 254), (0,0,0,0))
        avatar = self.profile.player.avatar.icon
            
        avatar,font_20,desc_frame = await asyncio.gather(pill.get_dowload_img(avatar, size= (134,134)),pill.get_font(18),_of.desc_frame)
            
        self.background_profile_avatar.alpha_composite(avatar)
        
        d = ImageDraw.Draw(self.background_profile_avatar)
        
        level = f"{self.lang.lvl}: {self.profile.player.level}"
        Wlevel = f"{self.lang.WL}: {self.profile.player.world_level}"
        if self.hide:
            uid  = 'UID: Hide'
        else:
            uid = f"UID: {self.uid}"
        
        d.text((134,4), self.profile.player.nickname, font= font_20, fill=(255,255,255,255))
        d.text((134,41), level, font= font_20, fill=(255,255,255,255))
        d.text((134,78), uid, font= font_20, fill=(255,255,255,255))
        d.text((134,115), Wlevel, font= font_20, fill=(255,255,255,255))
                
        signature = await pill.create_image_with_text(self.profile.player.signature, 18, max_width=606, color=(255, 255, 255, 255))
        self.background_profile_avatar.alpha_composite(desc_frame,(0,163))
        self.background_profile_avatar.alpha_composite(signature,(int(320-signature.size[0]/2),int(199-signature.size[1]/2)))
        
    
    async def build(self):
        
        self.background_profile.alpha_composite(self.background_profile_avatar,(29,53))

        x,y = 0,407  
        for i, key in enumerate(self.charter):
            self.background_profile.alpha_composite(key,(x,y))
            y += 155
            if i == 3:
                x = 338
                y = 407
                
        if not self.remove_logo:
            logo = await _of.LOGO_GIT_INV
            self.background_profile.alpha_composite(logo,(752,0))

                
    async def start(self):
        
        _of.set_mapping(3)
        
        self.font_charter = await pill.get_font(13)
        
        await asyncio.gather(self.creat_background(), self.get_charter(), self.creat_avatar())
        await self.build()        
        
        return self.background_profile


# Copyright 2024 DEViantUa <t.me/deviant_ua>
# All rights reserved.

import aiohttp
import asyncio
import json

class AioSession:
    _session = None
    _semaphore = asyncio.Semaphore(5)
    
    @classmethod
    async def creat_session(cls):
        """Creates a session
        Returns:
            aiohttp.ClientSession: The session instance.
        """
        cls._session = aiohttp.ClientSession()
        
        return cls._session
        
    @classmethod
    async def get_session(cls):
        """
        Returns the session instance, creating it if necessary.

        Returns:
            aiohttp.ClientSession: The session instance.
        """
        if cls._session is None:
            cls._session = aiohttp.ClientSession()
        return cls._session

    @classmethod
    async def close_session(cls):
        """
        Closes the session if it exists.
        """
        if cls._session is not None:
            await cls._session.close()
            cls._session = None

    @classmethod
    def session(cls):
        """
        Returns the current session instance.

        Returns:
            aiohttp.ClientSession: The current session instance, or None if the session is not created.
        """
        return cls._session

    @classmethod
    async def get(cls, url, headers=None, response_format='json'):
        """
        Sends a GET request using the current session instance.

        Args:
            url (str): The URL to send the request to.
            headers (dict): Optional headers to include in the request.
            response_format (str): The format of the response data to return.

        Returns:
            Depends on the specified response format:
            - dict: for 'json' format;
            - str: for 'text' format;
            - bytes: for 'bytes' format.
        """
        response = None
        creat_session = False
        try:
            if cls.session() is None:
                await cls.get_session()
                creat_session = True
            if cls._session is not None:
                async with cls._session.get(url, headers=headers) as response:
                    data = await cls.process_response(response, response_format)
                    if creat_session:
                        await cls.close_session()
                    return data
        except Exception as e:
            print(f"Error during GET request: {e}")
        finally:
            if response:
                await response.release()

    @classmethod
    async def post(cls, url, data=None, headers=None, response_format='json'):
        """
        Sends a POST request using the current session instance.

        Args:
            url (str): The URL to send the request to.
            data: The data to send in the request body.
            headers (dict): Optional headers to include in the request.
            response_format (str): The format of the response data to return.

        Returns:
            Depends on the specified response format:
            - dict: for 'json' format;
            - str: for 'text' format;
            - bytes: for 'bytes' format.
        """
        response = None
        try:
            if cls._session is not None:
                async with cls._session.post(url, data=data, headers=headers) as response:
                    data = await cls.process_response(response, response_format)
                    return data
        except Exception as e:
            print(f"Error during POST request: {e}")
        finally:
            if response:
                await response.release()

    @classmethod
    async def request(cls, method, url, headers=None, response_format='json', **kwargs):
        """
        Sends a custom HTTP request using the current session instance.

        Args:
            method (str): The HTTP method to use (e.g., 'GET', 'POST').
            url (str): The URL to send the request to.
            headers (dict): Optional headers to include in the request.
            response_format (str): The format of the response data to return.
            **kwargs: Additional keyword arguments to pass to the request function.

        Returns:
            Depends on the specified response format:
            - dict: for 'json' format;
            - str: for 'text' format;
            - bytes: for 'bytes' format.
        """
        response = None
        try:
            if cls._session is not None:
                async with cls._session.request(method, url, headers=headers, **kwargs) as response:
                    data = await cls.process_response(response, response_format)
                    return data
        except Exception as e:
            print(f"Error during custom request: {e}")
        finally:
            if response:
                await response.release()
   
    
    @classmethod
    async def process_response(cls, response, response_format):
        """
        Processes the response result before returning it in the specified format.

        Args:
            response (aiohttp.ClientResponse): The response object from the server.
            response_format (str): The format of the response data to return.

        Returns:
            Depends on the specified response format:
            - dict: for 'json' format;
            - str: for 'text' format;
            - bytes: for 'bytes' format.
        """
        if response_format == 'json':
            try:
                return await response.json()
            except aiohttp.ContentTypeError:
                data = await response.text()
                return json.loads(data)
        elif response_format == 'text':
            return await response.text()
        elif response_format == 'bytes':
            return await response.read()
        else:
            raise ValueError("Unsupported response format. Supported formats are 'json', 'text', and 'bytes'.")



# Copyright 2024 DEViantUa <t.me/deviant_ua>
# All rights reserved.

from PIL import Image,ImageDraw
from .. import git, options, treePaths
from .text_controle import get_font, create_image_with_text
from .image_controle import get_dowload_img
from .color_controle import  recolor_image, apply_opacity



async def creat_stats(combined_attributes, dop, 
                      font_dop,font,
                      line_size, name_size, max_width, icon_size,
                      icon_position, name_position, x_position,
                      color_font, stat_value_font,
                      stat_y_no_dop = 13, stat_y_yes_dop = 6, stat_y_dop = 25):
    
    font = await get_font(font)
    font_dop = await get_font(font_dop)
    stat_value_font = await get_font(stat_value_font)
    
    for i, attribute in enumerate(combined_attributes):
        stat = combined_attributes[attribute]
        icon = await get_dowload_img(stat.icon, size= icon_size)
        background = Image.new("RGBA", line_size, (0, 0, 0, 0))
        d = ImageDraw.Draw(background)
        value = "{:.1f}%".format(stat.value * 100) if stat.percent else round(stat.value)
        name_text = await create_image_with_text(stat.name, name_size, max_width=max_width, color=(255, 255, 255, 255)) #max_height=45,
        background.alpha_composite(name_text, (name_position[0],(int(name_position[1]-name_text.size[1]/2))))
        background.alpha_composite(icon,icon_position)
        
        if attribute in dop: 
            x = x_position - int(stat_value_font.getlength(str(value)))
            d.text((x, stat_y_yes_dop), str(value), font=stat_value_font, fill=(255, 255, 255, 255))

            x = x_position - int(font_dop.getlength(dop[attribute]["dop"]))
            d.text((x, stat_y_dop), dop[attribute]["dop"], font=font_dop, fill= color_font)

            x = x - int(font_dop.getlength(dop[attribute]["main"])) - 5
            d.text((x, stat_y_dop), dop[attribute]["main"], font=font_dop, fill=(255, 255, 255, 255))
        else:
            x = x_position - int(stat_value_font.getlength(str(value)))
            d.text((x, stat_y_no_dop), str(value), font=stat_value_font, fill=(255, 255, 255, 255))

        yield i, background
        

async def creat_path(character):
    background_path = await options.get_background_path(character.path.id)
    path = treePaths.map_new.get(character.path.id)
    font_15 = await get_font(15) 
    for key in character.skill_trees:
        if key.anchor in ['Point01','Point02','Point03','Point04','Point05']:
            icon = await get_dowload_img(key.icon, size=(47,47))
            icon = await recolor_image(icon, (255,212,173))
            count = await git.ImageCache().path_count
            count = count.copy()
            d = ImageDraw.Draw(count)
            if key.anchor == 'Point01':
                if key.max_level > 6:
                    key.level += 1
            else:
                if key.max_level > 10:
                    key.level += 2
                
            x = 29 - int(font_15.getlength(f"{key.level}/{key.max_level}")/2)
            d.text((x, 1), f"{key.level}/{key.max_level}", font=font_15, fill = (255,255,255,255))
            background_path.alpha_composite(icon, path[key.anchor]["icon"])
            background_path.alpha_composite(count, path[key.anchor]["count"])
        elif key.anchor in ['Point06','Point07','Point08']:
            icon = await get_dowload_img(key.icon, size=(47,47))
            icon = await recolor_image(icon, (0,0,0))
            if key.level == key.max_level:
                background_path.alpha_composite(icon, path[key.anchor]["icon"])
            else:
                icon = await apply_opacity(icon, opacity=0.5)
                closed = await git.ImageCache().path_closed_main
                background_path.alpha_composite(icon, path[key.anchor]["icon"])
                background_path.alpha_composite(closed, path[key.anchor]["closed"])
        else:
            icon = await get_dowload_img(key.icon, size=(35,35))
            icon = await recolor_image(icon, (0,0,0))
            if key.level == key.max_level:
                background_path.alpha_composite(icon, path[key.anchor]["icon"])
            else:
                icon = await apply_opacity(icon, opacity=0.5)
                closed = await git.ImageCache().path_closed_dop
                background_path.alpha_composite(icon, path[key.anchor]["icon"])
                background_path.alpha_composite(closed, path[key.anchor]["icon"])
                
    return background_path


_of = git.ImageCache()
_of.set_mapping(2)

async def get_resurs_light_cones(x):
    if x == 3:
        return await _of.shadow_3_light_cone, await _of.star_3_frame_light_cone, (150, 202, 255, 255)
    elif x == 4:
        return await _of.shadow_4_light_cone, await _of.star_4_frame_light_cone, (217, 150, 255, 255)
    else:
        return await _of.shadow_5_light_cone, await _of.star_5_frame_light_cone, (255, 199, 150, 255)
    
    
